#include "colorstyle.h"
#include "randomizer.h"  //For the cRandomizer in the mutate method
#include "polygon.h" //for the MF_ flags.

const Real cColorStyle::LW_ONEPIXEL = 0.0;
 //Color constants
const cVector3 cColorStyle::CN_RED = cVector3(255, 0, 0);
const cVector3 cColorStyle::CN_YELLOW = cVector3(255, 255, 0);
const cVector3 cColorStyle::CN_GREEN = cVector3(0, 255, 0);
const cVector3 cColorStyle::CN_BLUE = cVector3(0, 0, 255);
const cVector3 cColorStyle::CN_PURPLE = cVector3(255, 0, 255);
const cVector3 cColorStyle::CN_LIGHTRED = cVector3(255, 200, 200);
const cVector3 cColorStyle::CN_LIGHTYELLOW = cVector3(255, 255, 128);
const cVector3 cColorStyle::CN_ORANGE = cVector3(255, 128, 128);
const cVector3 cColorStyle::CN_LIGHTGREEN = cVector3(128, 255, 128);
const cVector3 cColorStyle::CN_LIGHTBLUE = cVector3(128, 128, 255);
const cVector3 cColorStyle::CN_LIGHTPURPLE = cVector3(255, 128, 255);
const cVector3 cColorStyle::CN_BLACK = cVector3(0,0,0);
const cVector3 cColorStyle::CN_WHITE = cVector3(255,255,255);
const cVector3 cColorStyle::CN_GRAY = cVector3(80,80,80);
const cVector3 cColorStyle::CN_LIGHTGRAY = cVector3(160,160,160);
const cVector3 cColorStyle::CN_BROWN = cVector3(128,128, 0);

cColorStyle::cColorStyle(const cColorStyle &colorstyle)
{
	copy(colorstyle);
}

cColorStyle::cColorStyle(cColorStyle *pcolorstyle)
{
	copy(*pcolorstyle);
}

void cColorStyle::copy(const cColorStyle &colorstyle)
{
	_filled = colorstyle._filled;
	_edged = colorstyle._edged;
	_fillcolor = colorstyle._fillcolor;
	_linecolor = colorstyle._linecolor;
	_linewidth = colorstyle._linewidth;
}


void cColorStyle::randomize(int mutationflags)
{
//The colors
	if (mutationflags & cPolygon::MF_COLOR)
	{
		_fillcolor = cRandomizer::pinst()->randomColor(0.3);
		_linecolor = cRandomizer::pinst()->randomColor(0.6);
	}
//The filling
	if (mutationflags & cPolygon::MF_FILLING)
		_filled = cRandomizer::pinst()->randomBool(0.85); 
	if (!_filled) //Don't allow invisibility!
		_edged = true;
	if (_filled && mutationflags & cPolygon::MF_EDGED)
		_edged =  cRandomizer::pinst()->randomBool(0.9); 
//The linewidth
	if (mutationflags & cPolygon::MF_LINEWIDTH)
	{
		if (cRandomizer::pinst()->randomBool(0.8)) //Most lines normal width
			_linewidth = 2.0;
		else
			_linewidth = cRandomizer::pinst()->
				randomReal(3.0, 20.0);
	}
}

//Global methods

void setVertexColor(cVector3 color, float alpha)
{
	float r = (float)color.x();
	float g = (float)color.y();
	float b = (float)color.z();
	CLAMP(r, 0.0, 1.0); //Use a macro from realnumber.h
	CLAMP(g, 0.0, 1.0);
	CLAMP(b, 0.0, 1.0);
	::glColor4f(r, g, b, alpha);
}	

void setVertexColorDim(cVector3 color, float dimfactor, float alpha)
{
	float r = dimfactor * (float)color.x();
	float g = dimfactor * (float)color.y();
	float b = dimfactor * (float)color.z();
	CLAMP(r, 0.0, 1.0); //Use a macro from realnumber.h
	CLAMP(g, 0.0, 1.0);
	CLAMP(b, 0.0, 1.0);
	::glColor4f(r, g, b, alpha);
}	

void setMaterialColor(cVector3 color)
{
  	float r = (float)color.x();
	float g = (float)color.y();
	float b = (float)color.z();
	CLAMP(r, 0.0, 1.0); //Use a macro from realnumber.h
	CLAMP(g, 0.0, 1.0);
	CLAMP(b, 0.0, 1.0);
	GLfloat materialAmbDiff[4]={ r, g, b, 1.0f };
	::glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, materialAmbDiff );
}

void setMaterialColorDim(cVector3 color, float dimfactor)
{
  	float r = dimfactor * (float)color.x();
	float g = dimfactor * (float)color.y();
	float b = dimfactor * (float)color.z();
	CLAMP(r, 0.0, 1.0); //Use a macro from realnumber.h
	CLAMP(g, 0.0, 1.0);
	CLAMP(b, 0.0, 1.0);
	GLfloat materialAmbDiff[4]={ r, g, b, 1.0f };
	::glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, materialAmbDiff );
}


