#ifndef COLORSTYLE_H
#define COLORSTYLE_H

#include "realnumber.h"
#include "vectormatrix.h"

class cColorStyle
{
public:
	static Real LINEWIDTHWEIGHT; //Default linewidth
	static const Real LW_ONEPIXEL; //Used as a linewidth value.
//Color constants.
	static const cVector3  CN_RED;
	static const cVector3  CN_YELLOW;
	static const cVector3  CN_GREEN;
	static const cVector3  CN_BLUE;
	static const cVector3  CN_PURPLE;
	static const cVector3  CN_LIGHTRED;
	static const cVector3  CN_LIGHTYELLOW;
	static const cVector3  CN_ORANGE;
	static const cVector3  CN_LIGHTGREEN;
	static const cVector3  CN_LIGHTBLUE;
	static const cVector3  CN_LIGHTPURPLE;
	static const cVector3  CN_BLACK;
	static const cVector3  CN_WHITE;
	static const cVector3  CN_GRAY;
	static const cVector3  CN_LIGHTGRAY;
	static const cVector3  CN_BROWN;
protected:
	bool _filled;
	bool _edged;
	cVector3 _fillcolor;
	cVector3 _linecolor;
	Real _linewidth;
public:
//Construct and copy
	cColorStyle(bool filled = TRUE, bool edged = TRUE, cVector3 fillcolor = CN_RED,
		cVector3 linecolor = CN_BLACK, Real linewidth = LW_ONEPIXEL):
		_filled(filled),_edged(edged),_fillcolor(fillcolor),
		_linecolor(linecolor),_linewidth(linewidth){}
	cColorStyle(const cColorStyle &colorstyle);
	cColorStyle(cColorStyle *pcolorstyle);
	void copy(const cColorStyle &colorstyle);
	cColorStyle* clone();
	cColorStyle operator=(const cColorStyle &colorstyle);
//Mutate
	void setFilled(bool yesno){_filled = yesno;}
	void setEdged(bool yesno){_edged = yesno;}
	virtual void setFillColor(cVector3 col){_fillcolor = col;}
	virtual void setLineColor(cVector3 col){_linecolor = col;}
	void setLineWidth(Real linewidth){_linewidth = linewidth;}
	void randomize(int mutationflags);
//Accessor
	bool filled() const {return _filled;}
	bool edged() const {return _edged;}
	cVector3 fillColor() const {return _fillcolor;}
	cVector3 lineColor() const {return _linecolor;}
	Real lineWidth() const {return _linewidth;}
};

//Global Methods
void setMaterialColor(cVector3 color); 
void setVertexColor(cVector3 color, float alpha = 1.0);
void setVertexColorDim(cVector3 color, float dimfactor = 0.5, float alpha = 1.0);
		 //Set a color but reduce all intensity by dimfactor.
void setMaterialColorDim(cVector3 color, float dimfactor = 0.5);

#endif //COLORSTYLE_H