#ifndef POLYGON_H
#define POLYGON_H

/*	
	1994.  The cPolygon class originally maintained a polygon as a "hand-made"
resizable array of cVector vertices.  It maintained an equally large hand-made
resizable array of CPoint pixel vertices as to support a 
realToPixel(CRealPixelConverter &) function which is called before using a
draw(CDC *) function to put the polygon on the screen.  The  CPoint _pointvert
array is used only in realToPixel and draw, we don't maintain its correctness
as we go along, only focussing on keeping the correct geometry of the polygon in
the cVector array.
	June 20, 1999.  The hand-made arrays were replaced with CArrays. The "dot"
feature was added to draw circles at the vertices.  Note that the dots as
implemented would not have the correct appearance for representing cPolygon3
projections. RR.
	August 4, 1999.  Changed it to a child of the cSprite class.
	June, 2001.  Gave it a general cVector which can be thought of as three-dimenisonal.
	Sept, 2003.  Moved it into standard C++ for use with GLUT in CS116A. Removed dot
feature.  Migrated OpenGL code into the cpp.
*/

#include "colorstyle.h"
#include <vector> //For "vector" template used for arrays
using namespace std; //Need this for standard template library to work.

class cPolygon 
{
public:
//const statics 
	//The MF_ flags are for use in mutating and randomizing
	static const int MF_RADIUS;
	static const int MF_ROTATION;
	static const int MF_COLOR;
	static const int MF_FILLING;
	static const int MF_EDGED;
	static const int MF_LINEWIDTH;
	static const int MF_VERTCOUNT;
	static const int MF_ALL; //MF_COLOR | MF_FILLING | MF_LINDWIDTH | DOTS 
		// | VERTCOUNT | MF_RADIUS | MF_ROTATION.
	static const int DF_WIREFRAME;
	static Real PRISMDZ; /* The default Z depth to use for a polygon sprite if we are
		in 3D and want to draw it with a thickness. */
	static Real cPolygon::MINRADIUS;
	static Real cPolygon::MAXRADIUS;
protected:
//These variables used to live in base class cSprite
	cColorStyle *_pcolorstyle; //To hold things like fillcolor, texture map, etc.
	Real _radius; /* Relates to the internal geometry of the sprite.  */
	cMatrix _attitude; /* We use _attitude as the modelview matrix. */
	cSpin _tumbleaxis;
	cMatrix _tumble; /* We tumble this much per step in animation. */
	cMatrix _precess; /* We rotate the tumble axis by this per step in animation. */
	Real _prismdz; /* _prismdz is a  variable that I may eventually use to
		give my polygons a 3D thickness. */
//Special cPolygon variables
	//Main array field and helper array field
	vector<cVector> _vectorvert;
	vector<cVector> _transformedvert; /* This is a helper for the
		call to transformedVert(const cMatrix &M) that we don't in fact use. */
	bool _closed; //Helper
//Private helper functions
	void _initializer(); //Used by the two constructors.
public:
//Constructor, destructor, operator=
	cPolygon(); //Default constructor calls initializer
	cPolygon(int vertcount); 
		//Makes a default regular polygon with n verts.
	cPolygon(int count, cVector *pverts, cColorStyle *pcolorstyle = NULL); 
		/* Useful for wrapping verts in a polygon class to draw. */
	virtual	~cPolygon(); /* The destructor of a base class must be virtual
		if any of its child classes can have more fields that may need additional
		destruction. Maybe use this some day. */
	void cPolygon::copy(const cPolygon &cpoly);
	cPolygon(const cPolygon &poly); //Uses copy
	cPolygon(cPolygon *ppoly); //Uses copy
	cPolygon operator=(const cPolygon &cpolygon); //uses the copy method
//Mutators
	//cColorstyle Mutators
	void setColorStyle(cColorStyle *pcolorstyle);
	void setFilled(bool yesno){_pcolorstyle->setFilled(yesno);}
	void setEdged(bool yesno){_pcolorstyle->setEdged(yesno);}
	void setFillColor(cVector3 col){_pcolorstyle->setFillColor(col);}
	void setLineColor(cVector3 col){_pcolorstyle->setLineColor(col);}
	void setLineWidth(Real linewidth){_pcolorstyle->setLineWidth(linewidth);}
	//Other Mutators
	void setPrismDz(Real prismdz){_prismdz = prismdz;}
	void randomize(int mutationflags);
	void setAttitude(const cMatrix &m){_attitude = m;}
	void setCenter(const cVector &newcenter){_attitude.setLastColumn(newcenter);}
	void setRadius(Real radius); /* May do more than set _radius. */
	void rotate(Real angle){_attitude*=cMatrix::zRotation(angle);}
	void rotate(const cSpin &spin){_attitude*=cMatrix::rotation(spin);}
	void translate(cVector trans){_attitude.translate(trans);}
	void fixCenterAndRadius(); /* Helper function calculates the centroid as the
		average of the vertices, moves the centroid the origin, and sets the _radius
		as the ax distance of a vertex from the origin. We plan to always center
		our polys at origin and use _attitude to move them */
	void setSize(int maxcount);//Allocates arrays, preserves old info.
	void setVertex (int n, const cVector &v);
	void addVertex (const cVector &v);
	void changeVertexcount(int updown); //Add updown to vert count.
	void setClosed(bool yesno){_closed = yesno;}
	void setRegularPolygon(int vertexcount,
		cVector center = cVector(0.0, 0.0, 0.0), Real radius = 1.0,
		Real initangle = 0.0);
	void setStarPolygon(int vertexcount, int step,
		cVector center = cVector(0.0, 0.0, 0.0), Real radius = 1.0,
		Real initangle = 0.0);
	void setRandomStarPolygon(int mincount, int maxcount);
	void setRandomRegularPolygon(int mincount, int maxcount);
	void setRandomAsteroidPolygon(int mincount = 5,	int maxcount = 30, Real spikiness = 0.3);
//Accessors
	bool closed() const {return _closed;}
	int vertCount() const {return _vectorvert.size();}  /*The const means the
		funtion doesn't change the class. I need this in case some
		functions that	call (const cPolygon &p) use the cPolygon::getSize function.*/
	cVector getVertex(int n) const;
	Real radius() const { return _radius * _attitude.scalefactor();}
	cVector center()const{return _attitude.lastColumn();}
	Real prismDz()const{return _prismdz;}
	cColorStyle* pcolorstyle(){return _pcolorstyle;}
		/* I'm going to lie and call this accessor const so I can use it in the const copy
		method that I need for the const copy constructor.  Calling it const is a lie because
		you can actually use a pcolorstyle() mutator to change fields. */
	cMatrix attitude()const{return _attitude;}
	Real angle();
	//cColorstyle accessors
	bool filled() const {return _pcolorstyle->filled();}
	bool edged() const {return _pcolorstyle->edged();}
	cVector3 fillColor() const {return _pcolorstyle->fillColor();}
	cVector3 lineColor() const {return _pcolorstyle->lineColor();}
	Real lineWidth() const {return _pcolorstyle->lineWidth();}
	cMatrix3 getTumble()const{return _tumble;}
//Matrix methods.
	vector<cVector>* transformedVert(const cMatrix &M); /* Fix our member array
		_transformedvert so it holds all the M*vert from the _vectorvert array,
		and then return a pointer to _transformedvert.  We can't return a copy of
		this array because we  may not have a vector<cVector> copy constructor. You 
		could use this if you didn't want to just let _attitude go into the OpenGL 
		pipeline. */
	friend cPolygon operator*(const cMatrix &M, cPolygon &p);
		/* Note that the cPolgon2 args can't be const because the copy
		constructor on a cPolygon can't be const. Even so, 
		we don't actually change anything in p.*/
//Most important methods
	void animateTumble(){_tumbleaxis *= _precess; _tumble = cMatrix::rotation(_tumbleaxis);_attitude *= _tumble;}
	void draw(int drawflags = 0); //Maybe later if drawflags is 1, we draw wireframe.
};
#endif //POLYGON_H