/* Sample code Snowstorm 3 by Rudy Rucker, CS Dept, SJSU, Sept 25, 2003.
This code is public domain and can be freely reused. */

#include <windows.h>   // use as needed for your system
#include <gl/Gl.h>  //Must include this before vectormatrix
#include <gl/glut.h>
#include <time.h> //For the clock() function used by sleep() for animation.
#include <math.h> //For cos and sine functions
#include "randomizer.h"
#include "vectormatrix.h"
#include "polygon.h"
#include <deque> //For "deque" template used for deques, which are double-ended queues
using namespace std; //Need this for standard template library to work.

cRandomizer RR;
GLdouble ZOOMRATIO = 0.8;
GLdouble startwindowwidth = 640.0;
GLdouble aspect = 4.0/3.0;
GLdouble xradius = 20.0;
GLdouble yradius = 15.0;
GLdouble zradius = 100.0;
cVector eye(0.0, - 0.05 * yradius, - zradius);
double zoom = 1.0;
GLdouble TIMEPERFRAME = 0.015; //Time in seconds per frame
boolean animateflag = true;
boolean threedflag = true;
int FLAKECOUNT = 300; //Number of flakes to have in the storm.
Real FALLSPEED = 0.3;

#define BLACKBACKGROUND
#define ANTIALIAS

//<<<<<<<<<<<<<<<<<<<<<<< myInit >>>>>>>>>>>>>>>>>>>>

void setProjectionMatrix(double zoom)
{
	glMatrixMode(GL_PROJECTION); 
	glLoadIdentity();
	if (!threedflag) //Use a 2D ortho projection.
		glOrtho(-zoom *xradius, zoom *xradius, -zoom *yradius, zoom *yradius,
			 -zradius, zradius); 
		/* Arguments: left, right, bottom, top, near, far.  The near and far are
		relative to a viewer located at the origin, so here we normally
		want a negative near and a positive far. We zoom only in the z and y,
		as zooming in the z might clip unnecessariy. */
	else //Use a 3D perspective projection
	{
		double boxradius = sqrt(xradius*xradius + yradius*yradius + zradius*zradius);
		double eyetoorigin = eye.magnitude();
		gluPerspective(zoom*45.0, aspect, 0.1, eyetoorigin+boxradius);
		/* Arguments: view angle in degrees, width-to-height aspect ratio, 
		near, far.  The aspect should match the viewport dx/dy.  The near
		and far should be both positive, as here we think of the eye as
		standing back from the scene. */
		gluLookAt(eye.x(), eye.y(), eye.z(), 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
		/* Arguments: 3 eye coords, 3 center coords, 3 up direction components.
		This puts the eye at the orgin of a viewer coordinate system and puts
		the "center" on the negative z axis. */
	}
}

void myInit(void)
{
/* We call myInit once at startup. */
#ifdef BLACKBACKGROUND
    glClearColor(0.0, 0.0, 0.0, 0.0);       // set black background color
#else //not BLACKBACKGROUND
    glClearColor(1.0, 1.0, 1.0, 0.0);       // set white background color
#endif //BLACKBACKGROUND
	setProjectionMatrix(1.0);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
//	glEnable(GL_LINE_SMOOTH);
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST); //In case I want to use 3D
}

//<<<<<<<<<<<<<<<<<<<<<<<< utilities >>>>>>>>>>>>>>>>>
void sleep(double secondstowait)
{
/* This makes the program spin in a loop for an interval of secondstowait.
It can make functionality a bit awkward for long secondstowait, as you aren't
polling the keyboard and mouse while sleep spins, which can make the
program feel dead.*/
	long waitticks = secondstowait * CLOCKS_PER_SEC; 
		//CLOCKS_PER_SEC is typically 30, it's a system thing.
	long goal = waitticks + clock();
	while( goal > clock() )
      ; //Spin a few thousand times.
}

//<<<<<<<<<<<<<<<<<<<<<<< drawFlurry >>>>>>>>>>>>>>>>>
void drawFlurry() //Uses the global afreq, bfreq, startt, dt, and tsteps params.
{
	static deque<cPolygon> polys;
	cPolygon newpoly;

	if(polys.size()>0)
		polys.pop_front(); //pop_front removes first (oldest) polygon from deque, returns void.
	while (polys.size() < FLAKECOUNT)
	{
		newpoly.setRandomRegularPolygon(3, 10);
		newpoly.randomize(cPolygon::MF_RADIUS | cPolygon::MF_ROTATION | cPolygon::MF_VERTCOUNT |
			cPolygon::MF_COLOR | cPolygon::MF_FILLING |	cPolygon::MF_LINEWIDTH |
			cPolygon::MF_EDGED); 
		newpoly.setCenter(cVector(RR.randomReal(-0.5* xradius, 0.5 * xradius),
			RR.randomReal(-0.5* yradius, 0.5* yradius),
			RR.randomReal(-zradius, zradius)));
		polys.push_back(newpoly); //push_back adds a copy of the polygon to the array.
	}
	for (int i=0; i<polys.size(); i++)
	{
		polys[i].animateTumble();
		polys[i].translate(-FALLSPEED * polys[i].radius() * cVector::ZAXIS);
		polys[i].draw();
		if (polys[i].center().z() < -zradius) //Once you're at the bottom stop moving.
			polys.erase(polys.begin()+i); //Erase when you hit bottom.
	}
}  
//<<<<<<<<<<<<<<<<<<<<<<<< myDisplay >>>>>>>>>>>>>>>>>
void myDisplay(void)
{
/* myDisplay is called once at start up and is called again whenever
the graphics window is first covered and then uncovered, making a
redraw necessary. */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);     // clear the screen 
//Draw 3D box around the world.
	glColor3f(0.8, 0.8, 0.8); //Light gray
	glLineWidth(1.0);
	glBegin(GL_LINE_LOOP); //loz face
		glVertex3d(-xradius, -yradius, -zradius);
		glVertex3d(xradius, -yradius, -zradius);
		glVertex3d(xradius, yradius, -zradius);
		glVertex3d(-xradius, yradius, -zradius);
	glEnd();
	glBegin(GL_LINE_LOOP); //hizface
		glVertex3d(-xradius, -yradius, zradius);
		glVertex3d(xradius, -yradius, zradius);
		glVertex3d(xradius, yradius, zradius);
		glVertex3d(-xradius, yradius, zradius);
	glEnd();
	glBegin(GL_LINE_LOOP); //loyface
		glVertex3d(-xradius, -yradius, zradius);
		glVertex3d(xradius, -yradius, zradius);
		glVertex3d(xradius, -yradius, -zradius);
		glVertex3d(-xradius, -yradius, -zradius);
	glEnd();
	glBegin(GL_LINE_LOOP); //hiyface
		glVertex3d(-xradius, yradius, zradius);
		glVertex3d(xradius, yradius, zradius);
		glVertex3d(xradius, yradius, -zradius);
		glVertex3d(-xradius, yradius, -zradius);
	glEnd();
	//Draw coordinate axes in Red, Blue, Yellow
	glLineWidth(3.0);
	glBegin(GL_LINES);
		glColor3f(1.0, 0.0, 0.0); //X axis red
		glVertex3d(0.0, 0.0, 0.0);
		glVertex3d(1.0, 0.0, 0.0);
		glColor3f(0.0, 0.0, 1.0); //Y axis blue
		glVertex3d(0.0, 0.0, 0.0);
		glVertex3d(0.0, 1.0, 0.0);
		glColor3f(1.0, 1.0, 0.0); //Z axis yellow
		glVertex3d(0.0, 0.0, 0.0);
		glVertex3d(0.0, 0.0, 1.0);
	glEnd();
	//Draw polygons
	drawFlurry();
	glFlush();				// Single buffer: send all output to display 
	glutSwapBuffers();		//Double buffer: flip.
}
//<<<<<<<<<<<<<<<<<<<<<<<< myReshape >>>>>>>>>>>>>>>>>

void myReshape(int width, int height)
{
	aspect = double(width)/double(height);
	setProjectionMatrix(zoom);
	glViewport(0, 0, width, height);
}
//<<<<<<<<<<<<<<<<<<<<<<<< myAnimate >>>>>>>>>>>>>>>>>
void myAnimate(void)
{
/* myAnimate is called over and over whenever glut is idle (not busy
processing a key or mouse message). To keep this from happening too
often we put in a delay, that is, we use a spin loop to make myAnimate
take at least TIMEPERFRAME seconds. */
	if (!animateflag)
		return;
	long waitticks = TIMEPERFRAME * CLOCKS_PER_SEC; 
		//CLOCKS_PER_SEC is typically 30 "ticks" per second, as I recall.
	long goal = waitticks + clock();
		//clock() returns a long integer that counts something like "ticks".
	myDisplay();
	while( goal > clock() )
      ; //Spin if there's unused time.
}
//<<<<<<<<<<<<<<<<<<<<<<<< myKeyboard >>>>>>>>>>>>>>>>>
void myKeyboard(unsigned char key, int mx, int my)
{
/* key is the ASCII code of the key, such as 'a', 'A', 'b', 'B', and
so on.  Note that case matters.  mx and my are the current mouse
position. */
	switch(key)
	{
		case ' ': //Space
			myDisplay(); //redraw 
			break;
		case 'a':
			animateflag ^= TRUE;
			break;
		case 'q':
			exit(-1); //Terminate
			break;
		case 'r':
			break;
		case 'z':
			zoom *= ZOOMRATIO;
			setProjectionMatrix(zoom);
			break;
		case 'Z':
			zoom *= 1.0/ZOOMRATIO;
			setProjectionMatrix(zoom);
			break;
		case '2':
			threedflag = false;
			setProjectionMatrix(zoom);
			break;
		case '3':
			threedflag = true;
			setProjectionMatrix(zoom);
			break;
	}
}
//<<<<<<<<<<<<<<<<<<<<<<<< myMouse >>>>>>>>>>>>>>>>>
void myMouse(int button, int state, int mx, int my)
{
/* Gets called when a button is clicked.  The arguments are:
button is GLUT_LEFT_BUTTON or GLUT_RIGHT_BUTTON,
state is GLUT_DOWN or GLUT_UP, 
and mx and my are window pixel coordinates, origin at window upper left. */
}
//<<<<<<<<<<<<<<<<<<<<<<<< myKeyboard >>>>>>>>>>>>>>>>>
void myMovedMouse(int mx, int my)
{
/*Gets called whenever the mouse moves, and mx and my are the new window
pixel coordinates, origin at window upper left. */
}
//<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>
void main(int argc, char** argv)
{
	glutInit(&argc, argv);          // initialize the toolkit
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB ); // set display mode
	glutInitWindowSize(startwindowwidth, int(startwindowwidth/aspect));     // set window size
	glutInitWindowPosition(100, 150); // set window position on screen
	glutCreateWindow("Snowstorm 4, Sept 29, 2003, Rudy Rucker"); // open the screen window
	glutDisplayFunc(myDisplay);     // register redraw function
	glutReshapeFunc(myReshape);
	glutKeyboardFunc(myKeyboard);	// register keyboard function.
	glutMouseFunc(myMouse);			// register mouse click function.
	glutMotionFunc(myMovedMouse);   // register mouse move function.
	glutIdleFunc(myAnimate);		// register idle function to call.
	myInit();                   
	glutMainLoop(); 		     // go into a perpetual loop
}
